"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.flexibleJson = void 0;
const ActionParameters_1 = require("../ActionParameters");
const secure_json_parse_1 = __importDefault(require("secure-json-parse"));
/**
 * Parses the first JSON object that it
 * finds in the response and is better suited for `gpt-3.5-turbo`, which does not
 * reliably insert the JSON object at the end of the response.
 */
const flexibleJson = () => ({
    description: "JSON",
    format(parameters) {
        return JSON.stringify(parameters, null, 2);
    },
    parse(text) {
        const [jsonObject, freeText] = extractFirstSingleLevelJsonObject(text);
        if (jsonObject == null) {
            return { _freeText: freeText };
        }
        try {
            return {
                ...ActionParameters_1.actionParametersSchema.parse(jsonObject),
                _freeText: freeText.trim(),
            };
        }
        catch (error) {
            throw new Error(`${text} could not be parsed as JSON: ${error?.message ?? error}`);
        }
    },
});
exports.flexibleJson = flexibleJson;
function extractFirstSingleLevelJsonObject(text) {
    const jsonStartIndex = text.indexOf("{");
    // assumes no nested objects:
    const jsonEndIndex = text.indexOf("}", jsonStartIndex);
    if (jsonStartIndex === -1 ||
        jsonEndIndex === -1 ||
        jsonStartIndex > jsonEndIndex) {
        return [null, text];
    }
    const jsonString = text.slice(jsonStartIndex, jsonEndIndex + 1);
    let jsonObject = null;
    try {
        jsonObject = secure_json_parse_1.default.parse(jsonString);
    }
    catch (error) {
        return [null, text];
    }
    const textBeforeJson = text.slice(0, jsonStartIndex);
    const textAfterJson = text.slice(jsonEndIndex + 1);
    return [jsonObject, textBeforeJson + textAfterJson];
}
//# sourceMappingURL=flexibleJson.js.map