"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AgentPlugin = void 0;
const zod_1 = __importDefault(require("zod"));
const ServerAgent_1 = require("./ServerAgent");
const AgentPlugin = ({ name, specification, }) => async function plugin(server) {
    const serverAgent = await ServerAgent_1.ServerAgent.create({
        specification,
    });
    const typedServer = server.withTypeProvider();
    // create agent run (POST /agent/:agent)
    typedServer.route({
        method: "POST",
        url: `/agent/${name}`,
        schema: {
            body: specification.inputSchema,
        },
        async handler(request, reply) {
            const runId = await serverAgent.createRun({
                input: request.body,
            });
            reply.code(201).send({ runId });
        },
    });
    // start agent run (POST /agent/:agent/run/:runId/start)
    typedServer.route({
        method: "POST",
        url: `/agent/${name}/run/:runId/start`,
        schema: {
            params: zod_1.default.object({
                runId: zod_1.default.string(),
            }),
        },
        async handler(request, reply) {
            const runId = request.params.runId;
            serverAgent.startRunWithoutWaiting({ runId });
            reply.code(201).send({ runId });
        },
    });
    // cancel agent run (POST /agent/:agent/run/:runId/cancel)
    typedServer.route({
        method: "POST",
        url: `/agent/${name}/run/:runId/cancel`,
        schema: {
            params: zod_1.default.object({
                runId: zod_1.default.string(),
            }),
            body: zod_1.default.object({
                reason: zod_1.default.string().optional(),
            }),
        },
        async handler(request, reply) {
            serverAgent.cancelRun({
                runId: request.params.runId,
                reason: request.body.reason,
            });
            reply.code(201).send({ runId: request.params.runId });
        },
    });
    // get agent run status:
    typedServer.route({
        method: "GET",
        url: `/agent/${name}/run/:runId`,
        schema: {
            params: zod_1.default.object({
                runId: zod_1.default.string(),
            }),
        },
        async handler(request, reply) {
            const runId = request.params.runId;
            const state = await serverAgent.getRunState({ runId });
            const accept = request.accepts();
            switch (accept.type(["json", "html"])) {
                case "json": {
                    reply.header("Content-Type", "application/json");
                    reply.send({ state });
                    break;
                }
                case "html": {
                    reply.header("Content-Type", "text/html");
                    reply.send(`
              <html>
                <head>
                  <title>Run ${runId}</title>
                </head>
                <body>
                  <pre>${JSON.stringify(state, null, 2)}</pre>
                </body>
              </html>
            `);
                    break;
                }
                default: {
                    reply.code(406).send({ error: "Not Acceptable" });
                    break;
                }
            }
        },
    });
};
exports.AgentPlugin = AgentPlugin;
//# sourceMappingURL=AgentPlugin.js.map