"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ServerAgent = void 0;
const hyperid_1 = __importDefault(require("hyperid"));
const Run_1 = require("../agent/Run");
const noLimit_1 = require("../agent/controller/noLimit");
const loadEnvironment_1 = require("../agent/env/loadEnvironment");
const controller_1 = require("../agent/controller");
const nextId = (0, hyperid_1.default)({ urlSafe: true });
class ServerAgent {
    static async create({ specification, }) {
        const environment = await (0, loadEnvironment_1.loadEnvironment)(specification.environment);
        const createRootStep = await specification.execute({ environment });
        return new ServerAgent({
            specification,
            environment,
            createRootStep,
        });
    }
    constructor({ specification, createRootStep, environment, }) {
        this.runs = new Map();
        this.specification = specification;
        this.environment = environment;
        this.createRootStep = createRootStep;
    }
    async createRun({ input }) {
        const dataProvider = this.specification.createDataProvider();
        const cancelController = (0, controller_1.cancellable)();
        const run = new Run_1.Run({
            controller: (0, controller_1.all)(this.specification.controller ?? (0, noLimit_1.noLimit)(), cancelController),
            initialState: await this.specification.init({
                environment: this.environment,
                input,
            }),
            observer: dataProvider,
        });
        const rootStep = await this.createRootStep(run);
        run.root = rootStep;
        const managedRun = new ManagedRun({
            id: nextId(),
            input,
            run,
            dataProvider,
            cancelController,
        });
        this.runs.set(managedRun.id, managedRun);
        return managedRun.id;
    }
    startRunWithoutWaiting({ runId }) {
        const run = this.runs.get(runId);
        if (run == null) {
            throw new Error(`Run ${runId} not found`);
        }
        // run asynchronously:
        setTimeout(async () => run.start(), 0);
    }
    cancelRun({ runId, reason }) {
        const run = this.runs.get(runId);
        if (run == null) {
            throw new Error(`Run ${runId} not found`);
        }
        run.cancel({ reason });
    }
    async getRunState({ runId }) {
        const run = this.runs.get(runId);
        if (run == null) {
            throw new Error(`Run ${runId} not found`);
        }
        return run.getState();
    }
}
exports.ServerAgent = ServerAgent;
class ManagedRun {
    constructor({ id, input, run, dataProvider, cancelController, }) {
        this.id = id;
        this.input = input;
        this.run = run;
        this.dataProvider = dataProvider;
        this.cancelController = cancelController;
    }
    async start() {
        this.run.onStart();
        const result = await this.run.root.execute();
        this.run.onFinish({ result });
    }
    cancel({ reason }) {
        this.cancelController.cancel({ reason });
    }
    async getState() {
        return {
            id: this.id,
            input: this.input,
            state: this.run.root.state,
            data: await this.dataProvider.getData({
                run: this.run,
            }),
        };
    }
}
//# sourceMappingURL=ServerAgent.js.map