"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.executeReadFile = exports.readFile = void 0;
const promises_1 = __importDefault(require("node:fs/promises"));
const node_path_1 = __importDefault(require("node:path"));
const zod_1 = __importDefault(require("zod"));
const readFile = ({ id = "read-file", description = "Read file content.", inputExample = {
    filePath: "{file path relative to the workspace folder}",
}, execute, formatResult = ({ summary, output: { content, error } }) => error
    ? `## ${summary}\n### Error\n${error}`
    : `## ${summary}\n### File content\n${content}`, }) => ({
    type: "basic-tool",
    id,
    description,
    inputSchema: zod_1.default.object({
        filePath: zod_1.default.string(),
    }),
    outputSchema: zod_1.default.object({
        content: zod_1.default.string().optional(),
        error: zod_1.default.string().optional(),
    }),
    inputExample,
    execute,
    formatResult,
});
exports.readFile = readFile;
const executeReadFile = ({ workspacePath, }) => async ({ input: { filePath } }) => {
    const fullPath = node_path_1.default.join(workspacePath, filePath);
    try {
        const content = await promises_1.default.readFile(fullPath, "utf-8");
        return {
            summary: `Read file ${filePath}`,
            output: { content },
        };
    }
    catch (error) {
        if (error.code === "ENOENT") {
            const { dir, files } = await getAvailableFiles(workspacePath, filePath);
            return {
                summary: `File not found.`,
                output: {
                    error: `Available files and directories in ${dir + node_path_1.default.sep}: ${files.join(", ")}`,
                },
            };
        }
        else {
            throw error;
        }
    }
};
exports.executeReadFile = executeReadFile;
async function getAvailableFiles(workspacePath, filePath) {
    // Find lowest existing directory
    const directories = filePath
        .split(node_path_1.default.sep)
        .slice(undefined, -1) // remove the file name
        .filter((directory) => directory !== "");
    let currentPath = "";
    for (const directory of directories) {
        try {
            await promises_1.default.access(node_path_1.default.join(workspacePath, currentPath, directory));
            currentPath = node_path_1.default.join(currentPath, directory);
        }
        catch (error) {
            if (error.code !== "ENOENT") {
                throw error;
            }
        }
    }
    // List files and directories
    const files = await promises_1.default.readdir(node_path_1.default.join(workspacePath, currentPath));
    return {
        dir: currentPath,
        files: await Promise.all(files.map(async (file) => {
            const stat = await promises_1.default.stat(node_path_1.default.resolve(workspacePath, file));
            return stat.isDirectory()
                ? node_path_1.default.join(currentPath, file) + node_path_1.default.sep
                : node_path_1.default.join(currentPath, file);
        })),
    };
}
//# sourceMappingURL=ReadFileTool.js.map